/********************************************
  lcd循环显示0～9实例程序
  每5秒变化一个数字，单数背光灯亮，双数背光灯熄灭
  显示位置：液晶左上角

  4位总线
  * LCD RS 连接到数字I/O PIN8
  * LCD Enable连接到数字I/O PIN9
  * LCD D4连接到数字I/O PIN4
  * LCD D5连接到数字I/O PIN5
  * LCD D6连接到数字I/O PIN6
  * LCD D7连接到数字I/O PIN7
  * LCD 背光灯控制使用数字I/O PIN10
  
  created 2011
  by Nille 
  Email：chenille@126.com
 
  This example code is in the public domain.
 *********************************************/

/*********************************************/
#define PortEn	9		//定义使能引脚
#define PortRS	8		//定义数据/命令选择引脚
#define PortBL	10		//定义背光控制引脚

/**********************************************
 写数据函数
函数功能：向1602液晶模块发送数据
入口参数：value——要发送的值
出口参数：无
**********************************************/
void lcdDat(uint8_t value)
{
  digitalWrite(PortRS,  HIGH);	//RS置高，发送的是数据
  digitalWrite(PortEn,  LOW);	//使能位拉低
	
  //先传送高4位
  for(int i = 4 ;i < 8 ; i++)
  digitalWrite( i, (value >> i) & 0x01);
	    
  //产生使能位下降沿
  digitalWrite(PortEn, HIGH);	
  digitalWrite(PortEn, LOW);
    
  //再传送低4位
  for(int i = 4 ;i < 8 ; i++)
  digitalWrite( i, (value >> i - 4) & 0x01);

  //产生使能位下降沿
  digitalWrite(PortEn, HIGH);
  digitalWrite(PortEn, LOW);
}

/**********************************************
写命令函数
函数功能：向1602液晶模块发送命令
入口参数：value——要发送的命令
出口参数：无
**********************************************/
void lcdCmd(uint8_t value)
{
  digitalWrite(PortRS, LOW);	//RS置低，发送的是命令
  digitalWrite(PortEn, LOW);	//使能位拉低
	
  //先传送高4位
  for(int i = 4 ;i < 8 ; i++)
  digitalWrite( i, (value >> i) & 0x01);
    
  //产生使能位下降沿
  digitalWrite(PortEn, HIGH);
  digitalWrite(PortEn, LOW);
    
  //再传送低4位
  for(int i = 4 ;i < 8 ; i++)
  digitalWrite( i, (value >> i - 4) & 0x01);

  //产生使能位下降沿
  digitalWrite(PortEn, HIGH);
  digitalWrite(PortEn, LOW);
}

/**********************************************
LCD初始化函数
函数功能：初始化1602液晶模块
入口参数：无
出口参数：无
**********************************************/
void lcdInit()
{
  delay(15);
  lcdCmd(0x28);	//功能设置，4位总线，双行显示
  delay(10);
  lcdCmd(0x28);	//功能设置，4位总线，双行显示
  delay(10);
  lcdCmd(0x28);	//功能设置，4位总线，双行显示
  delay(10);
  lcdCmd(0x01);	//清屏
  delay(10);
  lcdCmd(0x06);	//光标和显示模式设置
  delay(10);
  lcdCmd(0x0C);	//显示开，无光标
  delay(10);
}


/**********************************************
字符显示函数
函数功能：在1602液晶模块上显示字符
入口参数：pos——字符显示的位置
data——显示的字符
出口参数：无
**********************************************/
void lcdDisplay(uint8_t pos, uint8_t data)
{
  lcdCmd(pos | 0x80);	//设置显示字符的地址
  delay(10);
  lcdDat(data);	//发送要显示的数据
  delay(10);
}


/**********************************************
    初始化部分——setup函数
**********************************************/
void setup() 
{ 
  //设置引脚4～10为输出，控制1602液晶显示模块
  pinMode(4, OUTPUT);
  pinMode(5, OUTPUT);
  pinMode(6, OUTPUT);
  pinMode(7, OUTPUT);
  pinMode(8, OUTPUT);
  pinMode(9, OUTPUT);
  pinMode(10, OUTPUT);
  //初始化1602液晶显示模块
  lcdInit();
} 

/**********************************************
    执行部分——loop函数
**********************************************/
void loop() 
{ 
  for(int i='0';i<='9';i++)
  {
    //判断数字是单数还是偶数，决定背光灯点亮
    //单数背光灯亮，双数背光灯熄灭
    if(i%2 == 0)
    digitalWrite(PortBL, LOW);
    else
    digitalWrite(PortBL, HIGH);
		
    //在左上角（地址0x00）显示数字
    lcdDisplay( 0 , i );
    //延时5秒
    delay(5000);
  }
}
